use rocket_contrib::json::Json;
use rocket::{Response, State};
use rocket::http::Status;

use crate::config::Config;
use crate::DbConn;
use crate::models;


#[post("/users/me/token", data = "<auth_request>")]
pub async fn create_auth_token(
    conn: DbConn,
    config: State<'_, Config>,
    auth_request: Json<models::AuthTokenRequest>
) -> Result<Json<models::AuthTokenResponse>, models::ErrorResponse> {

    let user_info = conn.run(move |c| {
        models::LocalUser::get_user_by_creds(c, &auth_request.username, &auth_request.password)
    }).await?;

    let token = models::AuthClaims::new(&user_info, config.web_app.token_duration)
        .encode(&config.web_app.secret)
        .map_err(models::make_500)?;

    Ok(Json(models::AuthTokenResponse { token }))
}

#[post("/users", data = "<user_request>")]
pub async fn create_user<'r>(conn: DbConn, user_request: Json<models::CreateUserRequest>) -> Result<Response<'r>, models::ErrorResponse> {
    // TODO: Check current user if any to check if user has permission to create users (with or without role)
    conn.run(|c| {
        models::LocalUser::create_user(&c, user_request.into_inner())
    }).await?;

    Response::build()
        .status(Status::Created)
        .ok()
}
